==> demokritos-0.3.5.tgz: demokritos-0.3.5/lib/store.py <==
# Demokritos Atom Library and Server
# Copyright (C) 2005-2006 James Tauber
# See GPL file at top of distribution

"""
This is the core of the Atom store and ties together the service, workspace, 
collection and member domain objects.
"""

#import sys
#sys.path.append("lib")

from app import Service, Workspace, Collection, Member
from atom import Feed, Entry
from atom import Link, Date, Person
from atom import PlainText, InlineTextContent, OutOfLineContent
from app import parse_entry
from atom import generate_entry

from cStringIO import StringIO

import re
import time

class UnsupportedMediaTypeError(Exception):
    pass


class AtomStore:
    
    def __init__(self, root_uri, repos_path):
        """
        instantiates a new Atom Store with the given root URI.
        """
        
        self.root_uri = root_uri
        self.service = Service()

        # has to be here to avoid circularity
        from persistence import Persistence

        self.persistence = Persistence(repos_path, self)
        self.make_uri = UriMint(self.root_uri)
        
    def create_workspace(self, name):
        """
        creates an empty workspace.
        
        The workspace will have a name and will be registered with the store.
        """
        
        workspace = ServerWorkspace(name)
        self.service.workspaces.append(workspace)

        return workspace
    
    def create_collection(self, title, path, member_type):
        """
        creates an empty collection.
        
        The collection will have an ID and HREF based on the path provided as
        well as a title. The collection will also be registered with the store.
        
        Valid member types are currently "media" and "entry".
        """
        
        if member_type == "media":
            collection = ServerMediaCollection(title, path)
        elif member_type == "entry":
            collection = ServerEntryCollection(title, path)
        else:
            raise AttributeError, "unknown member_type"
            
        collection.href = self.root_uri + path
        collection.id = collection.href
        collection.store = self

        return collection

    def create_media_member(self, path):
        """
        creates an empty media member.
        
        The object will have an ID and HREF based on the path provided and will
        be registered with the store. It will not yet be in a collection
        however, and all other attributes will still need to be set.
        """

        member = ServerMediaMember(path)
        member.href = self.root_uri + path
        member.id = member.href

        return member

    def save_collection(self, collection):
        self.persistence.save_collection(collection)

    def save_member(self, member, collection):
        self.persistence.save_member(member, collection)
        # @@@ could be done explicitly
        # @@@ could be done in same transaction
        self.save_collection(collection)

    def create_entry_member(self, path):
        """
        creates an empty entry member.
        
        The object will have an ID and HREF based on the path provided and will
        be registered with the store. It will not yet be in a collection
        however, and all other attributes will still need to be set.
        """

        member = ServerEntryMember(path)
        member.href = self.root_uri + path
        member.id = member.href

        return member

    def is_collection(self, path):
        import persistence # @@@
        return self.persistence.get_type(path) == persistence.COLLECTION

    def is_member(self, path):
        import persistence # @@@
        return self.persistence.get_type(path) == persistence.MEMBER

    def get_collection(self, path):
        return self.persistence.get_collection(path)

    def get_member(self, path):
        return self.persistence.get_member(path)
 
    def remove_member(self, member):
        self.persistence.remove_member(member)

    def replace_member(self, old_member, content, media_type, suggested_title):
        collection = self.persistence.get_collection(old_member.collection_path)
        new_member = collection.create_member(content, media_type, suggested_title, old_member)

        return new_member

 
class UriMint:
    """
    This example of a URI Mint concatenates the collection path
    with yyyy/mm/dd with a modified suggested_name.
    """
    
    def __init__(self, root_uri):
        self.root_uri = root_uri
        
    def __call__(self, collection, suggested_name=None):
        if suggested_name:
            base_name = re.sub("[^A-Za-z0-9\-\.\,\(\)]", "_", suggested_name).lower()[:25]
        else:
            base_name = "resource" # @@@

        date = "/%04d/%02d/%02d/" % tuple(time.localtime()[:3])

        # @@@ doesn't check for duplicates yet

        # @@@ should support collection-specific prefix
        path = date + base_name
        uri = self.root_uri + path
        return uri, path


class ServerWorkspace(Workspace):
    
    def add_collection(self, collection):
        Workspace.add_collection(self, collection.title, collection.href, collection.member_type)


class ServerCollection(Collection):

    def __init__(self, title, path):
        Collection.__init__(self)
        self.title = title
        self.path = path
        self.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@
        self.member_type = None
        self.member_path_list = []

    def add_member(self, member):
        if not self.supported_media_type(member.media_type):
            raise UnsupportedMediaTypeError()

        Collection.add_member(self, member)
        member.collection_path = self.path
#        member.collections.add(self)
        self.updated = self.members[0].updated
        self.member_path_list = [member.path for member in self.members]
    
    def remove_member(self, member):
        Collection.remove_member(self, member)
        self.updated = self.members[0].updated
        self.member_path_list = [member.path for member in self.members]
    
    def make_feed(self, index=0):
    
        max_entries = 10 # @@@ magic number
        
        to = min(index + max_entries, len(self.members))
        
        if len(self.members) > to:
            next = to
        else:
            next = 0
        
        if index > 0:
            prev = min(0, index - max_entries)
            
        last = max(0, max_entries * ((len(self.members) - 1) / max_entries))
        
        feed = Feed()
        feed.id = self.id # @@@ 
        feed.title = PlainText(self.title) # @@@ perhaps Collection.title should be text construct
        feed.updated = self.updated
        
        if index:
            if index < 0 or index > len(self.members) - 1:
                raise AttributeError
            feed.links.append(Link(rel="first", href=self.href))
            if prev:
                feed.links.append(Link(rel="previous", href="%s?index=%d" % (self.href, prev)))
            else:
                feed.links.append(Link(rel="previous", href=self.href))
        if next:
            feed.links.append(Link(rel="next", href="%s?index=%d" % (self.href, next)))
            
        if last:
            feed.links.append(Link(rel="last", href="%s?index=%d" % (self.href, last)))
        else:
            # always include last
            feed.links.append(Link(rel="last", href=self.href))
                    
        feed.entries = [member.make_feed_entry() for member in self.members[index:to]]
        
        return feed
    


class ServerMediaCollection(ServerCollection):

    # @@@ add support for restricting media types
    
    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "media"

    def supported_media_type(self, media_type):
        return True

    def create_member(self, content, media_type, suggested_title, replace_member=None):
        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        if replace_member:
            uri = replace_member.href
            path = replace_member.path
        else:    
            uri, path = self.store.make_uri(self, suggested_title)

        member = self.store.create_media_member(path)
    
        member.title = PlainText(suggested_title or "no title") # @@@
        member.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@
        member.authors = [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated

        member.media_type = media_type
        member.raw_content = content
        
        self.add_member(member)
        
        self.store.save_member(member, self)
        
        return member
        

class ServerEntryCollection(ServerCollection):

    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "entry"

    def supported_media_type(self, media_type):
        # @@@ might need to be smarter than just startswith
        return media_type.startswith("application/atom+xml")

    def create_member(self, content, media_type, suggested_title, replace_member=None):
        # @@@ we don't currently do anything with a charset if it exists

        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        # may thrown AtomException
        entry = parse_entry(content)

        if replace_member:
            uri = replace_member.href
            path = replace_member.path
        else:    
            uri, path = self.store.make_uri(self, entry.title.value)

        member = self.store.create_entry_member(path)
        member.media_type = "application/atom+xml"

        entry.id = member.id
        entry.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()))
        
        if not entry.authors:
            entry.authors = [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated
        
        member.title = entry.title
        member.updated = entry.updated
        member.authors = entry.authors
        
        member.categories = entry.categories

        out = StringIO()
        generate_entry(out, entry, top=True)
        member.raw_content = out.getvalue()
        
        self.add_member(member)

        self.store.save_member(member, self)
        
        return member


class ServerMember(Member):

    def __init__(self, path):
        # Member.__init__(self)
        self.path = path
        self.collections = set()


class ServerMediaMember(ServerMember):
    
    def make_feed_entry(self):
        entry = Entry()
        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated
        # @@@ entry.published = self.published

        metadata_uri = self.href + "?atom" # @@@

        entry.links.append(Link(rel="edit", href=metadata_uri))

        entry.authors = self.authors
        # @@@ entry.contributors = self.contributors
        # @@@ entry.categories = self.categories
        
        entry.content = OutOfLineContent(src=self.href)
        
        return entry

    def get_content(self):
        return self.raw_content


class ServerEntryMember(ServerMember):
    
    def make_feed_entry(self):
        entry = Entry()

        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated

        if hasattr(self, "published"):
            entry.published = self.published

        # @@@ 
        entry.links.append(Link(rel="edit", href=self.href))

        entry.authors = self.authors
        
        if hasattr(self, "contributors"):
            entry.contributors = self.contributors
        
        if hasattr(self, "categories"):
            entry.categories = self.categories

        if hasattr(self, "summary"):
            entry.summary = self.summary

        entry.content = OutOfLineContent(src=self.href)

        return entry        
    
    def get_content(self):
        return self.raw_content

==> demokritos-0.3.7.tgz: demokritos-0.3.7/lib/store.py <==
# Demokritos Atom Library and Server
# Copyright (C) 2005-2006 James Tauber
# See GPL file at top of distribution

"""
This is the core of the Atom store and ties together the service, workspace, 
collection and member domain objects.
"""

#import sys
#sys.path.append("lib")

from app import Service, Workspace, Collection, Member
from atom import Feed, Entry
from atom import Link, Date, Person
from atom import PlainText, InlineTextContent, OutOfLineContent
from app import parse_entry
from atom import generate_entry

from cStringIO import StringIO

import re
import time

class UnsupportedMediaTypeError(Exception):
    pass

class MissingMediaTypeError(Exception):
    pass

class AtomStore:
    
    def __init__(self, root_uri, repos_path):
        """
        instantiates a new Atom Store with the given root URI.
        """
        
        self.root_uri = root_uri
        self.service = Service()

        # has to be here to avoid circularity
        from persistence import Persistence

        self.persistence = Persistence(repos_path, self)
        self.make_uri = UriMint(self, self.root_uri)
        
    def create_workspace(self, name):
        """
        creates an empty workspace.
        
        The workspace will have a name and will be registered with the store.
        """
        
        workspace = ServerWorkspace(name)
        self.service.workspaces.append(workspace)

        return workspace
    
    def create_collection(self, title, path, member_type):
        """
        creates an empty collection.
        
        The collection will have an ID and HREF based on the path provided as
        well as a title. The collection will also be registered with the store.
        
        Valid member types are currently "media" and "entry".
        """
        
        if member_type == "media":
            collection = ServerMediaCollection(title, path)
        elif member_type == "entry":
            collection = ServerEntryCollection(title, path)
        else:
            raise AttributeError, "unknown member_type"
            
        collection.href = self.root_uri + path
        collection.id = collection.href
        collection.store = self

        return collection

    def create_media_member(self, path):
        """
        creates an empty media member.
        
        The object will have an ID and HREF based on the path provided and will
        be registered with the store. It will not yet be in a collection
        however, and all other attributes will still need to be set.
        """

        member = ServerMediaMember(path)
        member.href = self.root_uri + path
        member.id = member.href

        return member

    def save_collection(self, collection):
        self.persistence.save_collection(collection)

    def save_member(self, member, collection):
        self.persistence.save_member(member, collection)
        # @@@ could be done explicitly
        # @@@ could be done in same transaction
        self.save_collection(collection)

    def create_entry_member(self, path):
        """
        creates an empty entry member.
        
        The object will have an ID and HREF based on the path provided and will
        be registered with the store. It will not yet be in a collection
        however, and all other attributes will still need to be set.
        """

        member = ServerEntryMember(path)
        member.href = self.root_uri + path
        member.id = member.href

        return member

    def is_collection(self, path):
        import persistence # @@@
        return self.persistence.get_type(path) == persistence.COLLECTION

    def is_member(self, path):
        import persistence # @@@
        return self.persistence.get_type(path) == persistence.MEMBER

    def exists(self, path):
        import persistence # @@@
        return self.persistence.get_type(path) != persistence.NONE

    def get_collection(self, path):
        return self.persistence.get_collection(path)

    def get_member(self, path):
        return self.persistence.get_member(path)
 
    def remove_member(self, member):
        self.persistence.remove_member(member)

    def replace_member(self, old_member, content, media_type, suggested_title):
        collection = self.persistence.get_collection(old_member.collection_path)
        new_member = collection.create_member(content, media_type, suggested_title, old_member)

        return new_member

 
class UriMint:
    """
    This example of a URI Mint concatenates the collection path
    with yyyy/mm/dd with a modified suggested_name.
    """
    
    def __init__(self, store, root_uri):
        self.store = store
        self.root_uri = root_uri
        
    def __call__(self, collection, suggested_name=None):
        if suggested_name:
            base_name = re.sub("[^A-Za-z0-9\-\.\,\(\)]", "_", suggested_name).lower()[:25]
        else:
            base_name = "resource" # @@@

        date = "/%04d/%02d/%02d/" % tuple(time.localtime()[:3])

        # @@@ doesn't check for duplicates yet

        # @@@ should support collection-specific prefix
        path = date + base_name

        orig_path = path
        count = 0
        
        while self.store.exists(path):
            count += 1
            path = orig_path + "_" + str(count)
        
        uri = self.root_uri + path
        return uri, path


class ServerWorkspace(Workspace):
    
    def add_collection(self, collection):
        Workspace.add_collection(self, collection.title, collection.href, collection.member_type)


class ServerCollection(Collection):

    def __init__(self, title, path):
        Collection.__init__(self)
        self.title = title
        self.path = path
        self.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@
        self.member_type = None
        self.member_path_list = []

    def add_member(self, member):
        if not self.supported_media_type(member.media_type):
            raise UnsupportedMediaTypeError()

        Collection.add_member(self, member)
        member.collection_path = self.path
#        member.collections.add(self)
        self.updated = self.members[0].updated
        self.member_path_list = [member.path for member in self.members]
    
    def remove_member(self, member):
        Collection.remove_member(self, member)
        self.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()))
        self.member_path_list = [member.path for member in self.members]
    
    def make_feed(self, index=0):
    
        max_entries = 10 # @@@ magic number
        
        to = min(index + max_entries, len(self.members))
        
        if len(self.members) > to:
            next = to
        else:
            next = 0
        
        if index > 0:
            prev = min(0, index - max_entries)
            
        last = max(0, max_entries * ((len(self.members) - 1) / max_entries))
        
        feed = Feed()
        feed.id = self.id # @@@ 
        feed.title = PlainText(self.title) # @@@ perhaps Collection.title should be text construct
        feed.updated = self.updated
        
        if index:
            if index < 0 or index > len(self.members) - 1:
                raise AttributeError
            feed.links.append(Link(rel="first", href=self.href))
            if prev:
                feed.links.append(Link(rel="previous", href="%s?index=%d" % (self.href, prev)))
            else:
                feed.links.append(Link(rel="previous", href=self.href))
        if next:
            feed.links.append(Link(rel="next", href="%s?index=%d" % (self.href, next)))
            
        if last:
            feed.links.append(Link(rel="last", href="%s?index=%d" % (self.href, last)))
        else:
            # always include last
            feed.links.append(Link(rel="last", href=self.href))
                    
        feed.entries = [member.make_feed_entry() for member in self.members[index:to]]
        
        return feed
    


class ServerMediaCollection(ServerCollection):

    # @@@ add support for restricting media types
    
    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "media"

    def supported_media_type(self, media_type):
        return True

    def create_member(self, content, media_type, suggested_title, replace_member=None):
        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        if replace_member:
            uri = replace_member.href
            path = replace_member.path
        else:    
            uri, path = self.store.make_uri(self, suggested_title)

        member = self.store.create_media_member(path)
    
        member.title = PlainText(suggested_title or "no title") # @@@
        member.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())) # @@@
        member.authors = [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated

        member.media_type = media_type
        member.raw_content = content
        
        self.add_member(member)
        
        self.store.save_member(member, self)
        
        return member
        

class ServerEntryCollection(ServerCollection):

    def __init__(self, title, path):
        ServerCollection.__init__(self, title, path)
        self.member_type = "entry"

    def supported_media_type(self, media_type):
        if media_type is None:
            raise MissingMediaTypeError()            
    
        # @@@ might need to be smarter than just startswith
        return media_type.startswith("application/atom+xml")

    def create_member(self, content, media_type, suggested_title, replace_member=None):
        # @@@ we don't currently do anything with a charset if it exists

        if not self.supported_media_type(media_type):
            raise UnsupportedMediaTypeError()

        # may thrown AtomException
        entry = parse_entry(content)

        if replace_member:
            uri = replace_member.href
            path = replace_member.path
        else:    
            uri, path = self.store.make_uri(self, entry.title.value)

        member = self.store.create_entry_member(path)
        member.media_type = "application/atom+xml"

        entry.id = member.id
        entry.updated = Date(time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime()))
        
        if not entry.authors:
            entry.authors = [Person(name="Sam Pell")] # @@@ eventually whoever is authenticated
        
        member.title = entry.title
        member.updated = entry.updated
        member.authors = entry.authors
        
        member.categories = entry.categories

        out = StringIO()
        generate_entry(out, entry, top=True)
        member.raw_content = out.getvalue()
        
        self.add_member(member)

        self.store.save_member(member, self)
        
        return member


class ServerMember(Member):

    def __init__(self, path):
        # Member.__init__(self)
        self.path = path
        self.collections = set()


class ServerMediaMember(ServerMember):
    
    def make_feed_entry(self):
        entry = Entry()
        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated
        # @@@ entry.published = self.published

        metadata_uri = self.href + "?atom" # @@@

        entry.links.append(Link(rel="edit", href=metadata_uri))

        entry.authors = self.authors
        # @@@ entry.contributors = self.contributors
        # @@@ entry.categories = self.categories
        
        entry.content = OutOfLineContent(src=self.href)
        
        return entry

    def get_content(self):
        return self.raw_content


class ServerEntryMember(ServerMember):
    
    def make_feed_entry(self):
        entry = Entry()

        entry.title = self.title
        entry.id = self.id
        entry.updated = self.updated

        if hasattr(self, "published"):
            entry.published = self.published

        # @@@ 
        entry.links.append(Link(rel="edit", href=self.href))

        entry.authors = self.authors
        
        if hasattr(self, "contributors"):
            entry.contributors = self.contributors
        
        if hasattr(self, "categories"):
            entry.categories = self.categories

        if hasattr(self, "summary"):
            entry.summary = self.summary

        entry.content = OutOfLineContent(src=self.href)

        return entry        
    
    def get_content(self):
        return self.raw_content
